// The MIT License

// Copyright (c) 2010 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Richard Uhler ruhler@mit.edu

import ClientServer::*;
import FixedPoint::*;
import GetPut::*;
import StmtFSM::*;


import Divider::*;

module test (Empty);

    Divider#(Bit#(48)) udiv <- mkNonRestoringDivider(4);
    Divider#(FixedPoint#(16, 16)) fpdiv <- mkFixedPointDivider(4);
    Reg#(Bool) failed <- mkReg(False);


    // Generate a single test case.
    // Warning: These are not parallel composable.
    // Input: n, d, expected q, expected r
    function Stmt atest(n, d, qe, re);
        return (seq
            udiv.request.put(tuple2(n, d));
            action
                match {.qa, .ra} <- udiv.response.get();
                if (qa != qe || ra != re) begin
                    $display("FAIL: %d/%d = %d r %d, but got %d r %d", n, d, qe, re, qa, ra);
                    failed <= True;
                end
            endaction
        endseq);
    endfunction

    // Generate a test case for the fixed point divider.
    // The remainder part is ignored.
    function Stmt fptest(FixedPoint#(16, 16) n, FixedPoint#(16, 16) d, FixedPoint#(16, 16) qe);
        return (seq
            fpdiv.request.put(tuple2(n, d));
            action
                match {.qa, .ra} <- fpdiv.response.get();
                if (qa != qe) begin
                    $write("FAIL: ");
                    fxptWrite(10, n);
                    $write("/");
                    fxptWrite(10, d);
                    $write(" = ");
                    fxptWrite(10, qe);
                    $write (" but got ");
                    fxptWrite(10, qa);
                    $display("");
                    failed <= True;
                end
            endaction
        endseq);
    endfunction

    Stmt dotest = (
        seq
            atest(10, 3, 3, 1);
            atest(2, 3, 0, 2);
            atest(1, 3, 0, 1);
            atest(0, 3, 0, 0);
            atest(3, 3, 1, 0);
            atest(4, 3, 1, 1);
            atest(14, 2, 7, 0);
            atest(4, 4, 1, 0);
            atest(13511, 234, 57, 173);

            fptest(10.0, 3.0, 3.3333333);
            fptest(-10.0, 3.0, -3.333333);
            fptest(10.0, -3.0, -3.333333);
            fptest(-10.0, -3.0, 3.333333);
            fptest(9.0, 3.0, 3.0);
            fptest(2.4, 1.2, 2.0);
            fptest(2.5, 1.2, 2.083333333);


            $display(failed ? "FAILED" : "PASSED");
        endseq
    );
    
    mkAutoFSM(dotest);
endmodule


